from typing import Optional, Any, Union

import requests
from xml.etree import ElementTree
import consts
from models import SessionLoginCap, SessionLogin
from helpers import sha256, xmlBuilder
from errors import errors
from datetime import datetime
import logging
import urllib.parse


_LOGGER = logging.getLogger(__name__)


class HikAxPro:
    """HikVisison Ax Pro Alarm panel coordinator."""

    host: str
    username: str
    password: str
    _cookie: Optional[str] = None

    def __init__(self, host: str, username: str, password: str):
        self.host = host
        self.username = username
        self.password = password

    def get_session_params(self):
        q_user = urllib.parse.quote(self.username)
        # q_password = urllib.parse.quote(self.password)
        # TODO needs basic auth ?
        # response = \
        #  requests.get(f"http://{q_user}:{q_password}@{self.host}{consts.Endpoints.Session_Capabilities}{q_user}")
        response = requests.get(f"http://{self.host}{consts.Endpoints.Session_Capabilities}{q_user}")
        _LOGGER.debug("Session_Capabilities response")
        _LOGGER.debug("Status: %s", response.status_code)
        _LOGGER.debug("Content: %s", response.content)
        _LOGGER.debug("Text: %s", response.text)
        _LOGGER.debug("Headers: %s", response.headers)
        _LOGGER.debug("End Session_Capabilities response")
        if response.status_code == 200:
            try:
                session_cap = self.parse_session_response(response.text)
                return session_cap
            except:
                raise errors.IncorrectResponseContentError()
        else:
            return None

    @staticmethod
    def set_logging_level(level: Union[int, str]):
        _LOGGER.setLevel(level)

    @staticmethod
    def _root_get_value(root: ElementTree, ns: dict, key: str, default: Optional[Any] = None) -> Optional[Any]:
        item = root.find(key, ns)
        if item is not None:
            return item.text
        return default

    @staticmethod
    def parse_session_response(xml_data):
        _LOGGER.debug("Debug data %s", xml_data)
        root = ElementTree.fromstring(xml_data)
        namespaces = {'xmlns': consts.XML_SCHEMA}

        session_id = HikAxPro._root_get_value(root, namespaces, "xmlns:sessionID")
        challenge = HikAxPro._root_get_value(root, namespaces, "xmlns:challenge")
        salt = HikAxPro._root_get_value(root, namespaces, "xmlns:salt")
        salt2 = HikAxPro._root_get_value(root, namespaces, "xmlns:salt2")
        is_irreversible = True if HikAxPro._root_get_value(root, namespaces, "xmlns:isIrreversible", False) == 'true' else False
        iterations = HikAxPro._root_get_value(root, namespaces, "xmlns:iterations")
        session_id_version = HikAxPro._root_get_value(root, namespaces, "xmlns:sessionIDVersion", None)
        if iterations is not None:
            iterations = int(iterations)
        session_cap = SessionLoginCap.SessionLoginCap(
            session_id=session_id,
            session_id_version=session_id_version,
            challenge=challenge,
            salt=salt,
            salt2=salt2,
            is_irreversible=is_irreversible,
            iterations=iterations
        )
        return session_cap

    def encode_password(self, session_cap: SessionLoginCap.SessionLoginCap):
        if session_cap.session_id_version == "2" and session_cap.is_irreversible:
            result = sha256.sha256(f"{self.username}{session_cap.salt}{self.password}")
            result = sha256.sha256(f"{result}{session_cap.challenge}")

            for i in range(2, session_cap.iterations):
                result = sha256.sha256(result)
        elif session_cap.is_irreversible:
            result = sha256.sha256(f"{self.username}{session_cap.salt}{self.password}")
            result = sha256.sha256(f"{self.username}{session_cap.salt2}{result}")
            result = sha256.sha256(f"{result}{session_cap.challenge}")

            for i in range(2, session_cap.iterations):
                result = sha256.sha256(result)
        else:
            result = f"{sha256.sha256(self.password)}{session_cap.challenge}"

            for i in range(1, session_cap.iterations):
                result = sha256.sha256(result)

        return result

    def connect(self):
        params = self.get_session_params()

        encoded_password = self.encode_password(params)

        xml = xmlBuilder.serialize_object(
            SessionLogin.SessionLogin(
                params.session_id,
                self.username,
                encoded_password,
                params.session_id_version
            )
        )

        dt = datetime.now()
        timestamp = datetime.timestamp(dt)
        session_login_url = f"http://{self.host}{consts.Endpoints.Session_Login}?timeStamp={int(timestamp)}"
        result = False
        try:
            login_response: requests.Response = requests.post(session_login_url, xml)
            _LOGGER.debug("Connect response")
            _LOGGER.debug("Status: %s", login_response.status_code)
            _LOGGER.debug("Content: %s", login_response.content)
            _LOGGER.debug("Text: %s", login_response.text)
            _LOGGER.debug("Headers: %s", login_response.headers)
            _LOGGER.debug("End connect response")
            if login_response.status_code == 200:
                cookie = login_response.headers.get("Set-Cookie")
                if cookie is None:
                    root = ElementTree.fromstring(login_response.text)
                    namespaces = {'xmlns': consts.XML_SCHEMA}
                    session_id = HikAxPro._root_get_value(root, namespaces, "xmlns:sessionID")
                    if session_id is not None:
                        cookie = "WebSession=" + session_id
                else:
                    self._cookie = cookie.split(";")[0]

                if cookie is None:
                    raise Exception("No cookie provided")

                self._cookie = cookie
                result = True
        except Exception as e:
            _LOGGER.error("Error in parsing response", exc_info=e)
            result = False
            self._cookie = None

        return result

    def logout(self):
        response = self._base_json_request(f"http://{self.host}{consts.Endpoints.Session_Logout}", method=consts.Method.PUT)
        if response.status_code != 200:
            raise errors.UnexpectedResponseCodeError(response.status_code, response.text)
        self._cookie = ''
        return response.status_code == 200

    def is_logged_in(self):
        return self._cookie is not None

    @staticmethod
    def build_url(endpoint: str, is_json: bool = False):
        param_prefix = "&" if "?" in endpoint else "?"
        return f"{endpoint}{param_prefix}format=json" if is_json else endpoint

    def _base_json_request(self, url: str, method: consts.Method = consts.Method.GET, data=None):
        endpoint = self.build_url(url, True)
        response = self.make_request(endpoint, method, is_json=True, data=data)

        if response.status_code != 200:
            raise errors.UnexpectedResponseCodeError(response.status_code, response.text)
        if response.status_code == 200:
            return response.json()

    def arm_home(self, sub_id: Optional[int] = None, code: Optional[str] = None):
        sid = "0xffffffff" if sub_id is None else str(sub_id)
        data = None
        if code is not None:
            data = {"Operate": {"moduleOperateCode": code}}
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.Alarm_ArmHome.replace('{}', sid)}",
                                       method=consts.Method.PUT, data=data)

    def arm_away(self, sub_id: Optional[int] = None, code: Optional[str] = None):
        sid = "0xffffffff" if sub_id is None else str(sub_id)
        data = None
        if code is not None:
            data = {"Operate": {"moduleOperateCode": code}}
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.Alarm_ArmAway.replace('{}', sid)}",
                                       method=consts.Method.PUT, data=data)

    def disarm(self, sub_id: Optional[int] = None, code: Optional[str] = None):
        sid = "0xffffffff" if sub_id is None else str(sub_id)
        data = None
        if code is not None:
            data = {"Operate": {"moduleOperateCode": code}}
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.Alarm_Disarm.replace('{}', sid)}",
                                       method=consts.Method.PUT, data=data)

    def subsystem_status(self):
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.SubSystemStatus}")

    def peripherals_status(self):
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.PeripheralsStatus}")

    def zone_status(self):
        endpoint = f"http://{self.host}{consts.Endpoints.ZoneStatus}"
        endpoint = self.build_url(endpoint, True)
        response = self.make_request(endpoint, consts.Method.GET)

        if response.status_code != 200:
            raise errors.UnexpectedResponseCodeError(response.status_code, response.text)

        return response.json()

    def bypass_zone(self, zone_id):
        endpoint = f"http://{self.host}{consts.Endpoints.BypassZone}{zone_id}"
        endpoint = self.build_url(endpoint, True)
        response = self.make_request(endpoint, consts.Method.PUT)

        if response.status_code != 200:
            raise errors.UnexpectedResponseCodeError(response.status_code, response.text)

        return response.status_code == 200

    def recover_bypass_zone(self, zone_id):
        endpoint = f"http://{self.host}{consts.Endpoints.RecoverBypassZone}{zone_id}"
        endpoint = self.build_url(endpoint, True)
        response = self.make_request(endpoint, consts.Method.PUT)

        return response.status_code == 200

    def get_interface_mac_address(self, interface_id):
        endpoint = f"http://{self.host}{consts.Endpoints.InterfaceInfo}"

        response = self.make_request(endpoint, consts.Method.GET)

        if response.status_code == 200:
            return xmlBuilder.get_mac_address_of_interface(response.text, interface_id)

        return ''

    def get_area_arm_status(self, area_id):
        endpoint = f"http://{self.host}{consts.Endpoints.AreaArmStatus}"
        endpoint = self.build_url(endpoint, True)

        data = {"SubSysList": [{"SubSys": {"id": area_id}}]}

        response = self.make_request(endpoint, consts.Method.POST, data=data, is_json=True)

        try:
            if response.status_code == 200:
                response_json = response.json()
                return response_json["ArmStatusList"][0]["ArmStatus"]["status"]
        except:
            return ''
        return ''

    def host_status(self):
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.HostStatus}")

    def siren_status(self):
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.SirenStatus}")

    def keypad_status(self):
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.KeypadStatus}")

    def repeater_status(self):
        return self._base_json_request(f"http://{self.host}{consts.Endpoints.RepeaterStatus}")

    def make_request(self, endpoint, method, data=None, is_json=False):
        headers = {"Cookie": self._cookie}

        if method == consts.Method.GET:
            response = requests.get(endpoint, headers=headers)
        elif method == consts.Method.POST:
            if is_json:
                response = requests.post(endpoint, json=data, headers=headers)
            else:
                response = requests.post(endpoint, data=data, headers=headers)
        elif method == consts.Method.PUT:
            if is_json:
                response = requests.put(endpoint, json=data, headers=headers)
            else:
                response = requests.put(endpoint, data=data, headers=headers)
        else:
            return None

        if response.status_code == 401:
            self.connect()
            response = self.make_request(endpoint, method, data, is_json)

        return response



class SessionLoginCap:
    def __init__(self, session_id, session_id_version, challenge, salt, salt2, is_irreversible, iterations):
        self.session_id = session_id
        self.session_id_version = session_id_version
        self.challenge = challenge
        self.salt = salt
        self.salt2 = salt2
        self.is_irreversible = is_irreversible
        self.iterations = iterations



class Endpoints:
    Session_Capabilities = "/ISAPI/Security/sessionLogin/capabilities?username="
    Session_Login = "/ISAPI/Security/sessionLogin"
    Session_Logout = "/ISAPI/Security/sessionLogout"
    Alarm_Disarm = "/ISAPI/SecurityCP/control/disarm/{}"
    Alarm_ArmAway = "/ISAPI/SecurityCP/control/arm/{}?ways=away"
    Alarm_ArmHome = "/ISAPI/SecurityCP/control/arm/{}?ways=stay"
    SubSystemStatus = "/ISAPI/SecurityCP/status/subSystems"
    AlertStream = "/ISAPI/Event/notification/alertStream"
    DetectorConfig = "/ISAPI/SecurityCP/BasicParam/DetectorCfg"
    DetectorConfigCap = "/ISAPI/SecurityCP/BasicParam/DetectorCfg/capabilities"
    Caps = "/ISAPI/SecurityCP/capabilities"
    CheckResultCap = "/ISAPI/SecurityCP/CheckResult/capabilities"
    CheckResult = "/ISAPI/SecurityCP/CheckResult"
    ConfCap = "/ISAPI/SecurityCP/Configuration/capabilities"
    ZonesConfig = "/ISAPI/SecurityCP/Configuration/zones"
    DeviceTime = "/ISAPI/SecurityCP/Configuration/deviceTime"
    EventRecordCap = "/ISAPI/SecurityCP/Configuration/eventRecord/channels/2/capabilities"
    EventRecord = "/ISAPI/SecurityCP/Configuration/eventRecord/channels/1"
    FaultCheck = "/ISAPI/SecurityCP/Configuration/faultCheckCfg"
    GlassBreakDetector = "/ISAPI/SecurityCP/Configuration/glassBreakDetector/zone/5"
    MagneticContact = "/ISAPI/SecurityCP/Configuration/magneticContact/zone/0"
    PublicSubSystem = "/ISAPI/SecurityCP/Configuration/publicSubSys"
    ZonesCap = "/ISAPI/SecurityCP/Configuration/zones/capabilities"
    Zones = "/ISAPI/SecurityCP/Configuration/zones/"
    ArmStatus = "/ISAPI/SecurityCP/status/armStatus"
    StatusCap = "/ISAPI/SecurityCP/status/capabilities"
    HostStatus = "/ISAPI/SecurityCP/status/host"  
    PeripheralsStatus = "/ISAPI/SecurityCP/status/exDevStatus"  
    ZoneStatus = "/ISAPI/SecurityCP/status/zones"
    BypassZone = "/ISAPI/SecurityCP/control/bypass/"
    RecoverBypassZone = "/ISAPI/SecurityCP/control/Recoverbypass/"
    InterfaceInfo = "/ISAPI/System/Network/interfaces"
    AreaArmStatus = "/ISAPI/SecurityCP/status/armStatus"
    SirenStatus = "/ISAPI/SecurityCP/status/sirenStatus"
    RepeaterStatus = "/ISAPI/SecurityCP/status/repeaterStatus"
    KeypadStatus = "/ISAPI/SecurityCP/status/keypadStatus"
    OutputStatus = "/ISAPI/SecurityCP/status/outputStatus"
    BatteriesStatus = "/ISAPI/SecurityCP/status/batteries"
    HostCapabilities = "/ISAPI/SecurityCP/control/capabilities"
    SystemDeviceInfo = "/ISAPI/System/deviceInfo"
    OutputConfig = "/ISAPI/SecurityCP/Configuration/outputs"
    SirensConfig = "/ISAPI/SecurityCP/Configuration/wirelessSiren"



class Method:
    GET = "GET"
    POST = "POST"
    PUT = "PUT"
    DELETE = "DELETE"


XML_SERIALIZABLE_NAMES = ["SessionLogin", "userName", "password", "sessionID", "sessionIDVersion"]


XML_SCHEMA = "http://www.hikvision.com/ver20/XMLSchema"



import pytest
import requests_mock
from src import hikaxpro
from src.models import SessionLoginCap
from src.helpers import xmlBuilder
from src import consts

@pytest.fixture(scope="session")
def theaxpro():
    theaxpro = hikaxpro.HikAxPro("192.168.72.136", "blabla@blabla.com", "blabla")
    return theaxpro

@requests_mock.Mocker(kw='mock')
def test_getSessionParams(theaxpro, **kwargs):        
    url = f"http://{theaxpro.host}{consts.Endpoints.Session_Capabilities}{theaxpro.username}"
    responseText = """<SessionLoginCap version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                            <sessionID>116c191b1981ca41fd110a4140e63c522c0d168a460fed22bc0b4ca84e18fe0e</sessionID>
                            <challenge>f0a902b12718652487db2b0cd9e83c4a</challenge><iterations>100</iterations>
                            <isSupportRTSPWithSession>true</isSupportRTSPWithSession>
                            <isIrreversible>true</isIrreversible>
                            <sessionIDVersion>2.1</sessionIDVersion>
                            <salt>22CF57B6ADE75214A4C87042B3272630C55EF5D782AB3BE4C9EC4DA1CD95AF3B</salt>
                            <salt2>7BEB8CA39D05B89CABC4003FDCBA5AE73556CB8008BCEE3CBCA48CABC3AC201B</salt2>
                        </SessionLoginCap>"""    

    kwargs["mock"].get(url, text=responseText, status_code=200)

    sessionCap = theaxpro.get_session_params()
    assert sessionCap is not None

def test_parseSessionResponse(theaxpro):
    sessionXml = """<SessionLoginCap version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                        <sessionID>2109bea576cb68f1c2fc7de994efae250de4e2d75c0856762e97429f1f8bce17</sessionID>
                        <challenge>ab1c5d159387de8388b097709e8fecf9</challenge>
                        <iterations>100</iterations>
                        <isSupportRTSPWithSession>true</isSupportRTSPWithSession>
                        <isIrreversible>true</isIrreversible>
                        <sessionIDVersion>2.1</sessionIDVersion>
                        <salt>22CF57B6ADE75214A4C87042B3272630C55EF5D782AB3BE4C9EC4DA1CD95AF3B</salt>
                        <salt2>7BEB8CA39D05B89CABC4003FDCBA5AE73556CB8008BCEE3CBCA48CABC3AC201B</salt2>
                    </SessionLoginCap>"""
    
    axProSession = theaxpro.parse_session_response(sessionXml)
    
    assert len(axProSession.session_id) > 0
    assert len(axProSession.challenge) > 0
    assert len(axProSession.salt) > 0
    assert len(axProSession.salt2) > 0

def test_encodePassword_Irrevesible(theaxpro):
    sessionID = ''
    challange = '82360cd6ffd7beda4398cf67bfbd1ea9'
    iteration = 100
    salt = '22CF57B6ADE75214A4C87042B3272630C55EF5D782AB3BE4C9EC4DA1CD95AF3B'
    salt2 = '7BEB8CA39D05B89CABC4003FDCBA5AE73556CB8008BCEE3CBCA48CABC3AC201B'
    sessionCap = SessionLoginCap.SessionLoginCap(sessionID, "2.1", challange, salt, salt2, True, iteration)

    encodedPassword = theaxpro.encode_password(sessionCap)

    assert encodedPassword == "777b95a40f8b5b9ca25101d3c73168adb340f2001b611a74104c34352962d647"


def test_encodePassword_Irrevesible_olderVersion():
    theaxpro = hikaxpro.HikAxPro("192.168.72.136", "dalferin", "test123456")
    sessionID = "2c9ac7c6d7002d6bc1359c59c7968fd6683569cf47ad149826cf1e5c84cbb7"
    challange = "9524eedd2912a46ef2c0fcfa27e412c1"
    iteration = 100
    salt = "bef6d541c1ac37f74d90b9d3dc642ab6ea54b17d653b10f24464867b6dbeacb5"
    salt2 = None
    sessionCap = SessionLoginCap.SessionLoginCap(sessionID, "2.1", challange, salt, salt2, True, iteration)

    encodedPassword = theaxpro.encode_password(sessionCap)

    assert encodedPassword == '86c59b2a28aafd52ccbd@a46f06200e50467cec0758eb32a8c5d5cce5b55abc'

def test_encodePassword_Not_Irreversible(theaxpro):
    sessionID = ''
    challange = '82360cd6ffd7beda4398cf67bfbd1ea9'
    iteration = 100
    salt = '22CF57B6ADE75214A4C87042B3272630C55EF5D782AB3BE4C9EC4DA1CD95AF3B'
    salt2 = '7BEB8CA39D05B89CABC4003FDCBA5AE73556CB8008BCEE3CBCA48CABC3AC201B'
    sessionCap = SessionLoginCap.SessionLoginCap(sessionID, "2", challange, salt, salt2, False, iteration)

    encodedPassword = theaxpro.encode_password(sessionCap)
    assert encodedPassword == "2b31a14ba59914a8e5e5510063e5d500ef322e76bc6c160df6a86b35c1e89ddd"

@requests_mock.Mocker(kw='mock')
def test_connect_successfull(theaxpro, **kwargs):  
    url = f"http://{theaxpro.host}{consts.Endpoints.Session_Capabilities}{theaxpro.username}"
    responseText = """<SessionLoginCap version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                            <sessionID>116c191b1981ca41fd110a4140e63c522c0d168a460fed22bc0b4ca84e18fe0e</sessionID>
                            <challenge>f0a902b12718652487db2b0cd9e83c4a</challenge><iterations>100</iterations>
                            <isSupportRTSPWithSession>true</isSupportRTSPWithSession>
                            <isIrreversible>true</isIrreversible>
                            <sessionIDVersion>2.1</sessionIDVersion>
                            <salt>22CF57B6ADE75214A4C87042B3272630C55EF5D782AB3BE4C9EC4DA1CD95AF3B</salt>
                            <salt2>7BEB8CA39D05B89CABC4003FDCBA5AE73556CB8008BCEE3CBCA48CABC3AC201B</salt2>
                        </SessionLoginCap>"""    

    kwargs["mock"].get(url, text=responseText, status_code=200)
    sessionLoginUrl = f"http://{theaxpro.host}{consts.Endpoints.Session_Login}" 
    kwargs["mock"].post(sessionLoginUrl, headers={"Set-Cookie": "blabla;bla"}, status_code=200)
    loginResult = theaxpro.connect()
    assert loginResult is True

def test_buildUrl_json(theaxpro):    
    url = theaxpro.build_url("http://blabla.com", True)
    assert url == "http://blabla.com?format=json"

def test_buildUrl_not_json(theaxpro):    
    url = theaxpro.build_url("http://blabla.com", False)
    assert url == "http://blabla.com"

@requests_mock.Mocker(kw='mock')
def test_arm_home(theaxpro, **kwargs):   
    url = f"http://{theaxpro.host}{consts.Endpoints.Alarm_ArmHome}"
    kwargs["mock"].put(url, status_code=200)
    result = theaxpro.arm_home()
    assert result is True

@pytest.mark.skip(reason="Integration test. Requires actual credentials.")
def test_arm_home_integration(theaxpro):
    url = url = f"http://{theaxpro.host}{consts.Endpoints.Alarm_ArmHome}"
    result = theaxpro.arm_home()
    assert result is True

@requests_mock.Mocker(kw='mock')
def test_arm_away(theaxpro, **kwargs):    
    url = f"http://{theaxpro.host}{consts.Endpoints.Alarm_ArmAway}"
    kwargs["mock"].put(url, status_code=200)
    result = theaxpro.arm_away()
    assert result is True

@pytest.mark.skip(reason="Integration test. Requires actual credentials.")
def test_arm_away_integration(theaxpro):
    url = f"http://{theaxpro.host}{consts.Endpoints.Alarm_ArmAway}"
    result = theaxpro.arm_away()
    assert result is True

@requests_mock.Mocker(kw='mock')
def test_disarm(theaxpro, **kwargs):    
    url = f"http://{theaxpro.host}{consts.Endpoints.Alarm_Disarm}"
    kwargs["mock"].put(url, status_code=200)
    result = theaxpro.disarm()
    assert result is True

@pytest.mark.skip(reason="Integration test. Requires actual credentials.")
def test_disarm_integration(theaxpro):
    url = f"http://{theaxpro.host}{consts.Endpoints.Alarm_Disarm}"
    result = theaxpro.disarm()
    assert result is True

@requests_mock.Mocker(kw='mock')
def test_subsystem_status(theaxpro, **kwargs):    
    url = f"http://{theaxpro.host}{consts.Endpoints.SubSystemStatus}"
    kwargs["mock"].get(url, json={"SubSysList":[]}, status_code=200)
    result = theaxpro.subsystem_status()
    assert ("SubSysList" in result) is True

@requests_mock.Mocker(kw='mock')
def test_peripherals_status_success(theaxpro, **kwargs):    
    url = f"http://{theaxpro.host}{consts.Endpoints.PeripheralsStatus}"    
    kwargs["mock"].get(url, json={ 'ExDevStatus': 'x'}, status_code=200)
    result = theaxpro.peripherals_status()
    assert ("ExDevStatus" in result) is True

@requests_mock.Mocker(kw='mock')
def test_zone_status_success(theaxpro, **kwargs):    
    url = f"http://{theaxpro.host}{consts.Endpoints.ZoneStatus}"
    responseJson = '{"ZoneList": []}'
    kwargs["mock"].get(url, json=responseJson, status_code=200)
    result = theaxpro.zone_status()
    assert ("ZoneList" in result) is True

@requests_mock.Mocker(kw='mock')
def test_bypass_zone(theaxpro, **kwargs):
    url = f"http://{theaxpro.host}{consts.Endpoints.BypassZone}1"
    kwargs["mock"].put(url, status_code=200)
    result = theaxpro.bypass_zone(1)
    assert result is True

@requests_mock.Mocker(kw='mock')
def test_recover_bypass_zone(theaxpro, **kwargs):
    url = f"http://{theaxpro.host}{consts.Endpoints.RecoverBypassZone}1"
    kwargs["mock"].put(url, status_code=200)
    result = theaxpro.recover_bypass_zone(1)
    assert result is True

@pytest.mark.skip(reason="Integration test. Requires actual credentials.")
def test_get_interface_mac_address():
    axpro = hikaxpro.HikAxPro('192.168.72.226', 'blabla@bla.com', 'blabla')
    mac_address = axpro.get_interface_mac_address(1)

    assert mac_address == '2c:a5:9c:cd:20:0a'

@pytest.mark.skip(reason="Integration test. Requires actual credentials.")
def test_get_area_arm_status():
    axpro = hikaxpro.HikAxPro('192.168.72.226', 'blabla@bla.com', 'blabla')
    status = axpro.get_area_arm_status(1)

    assert status == 'disarmed'


class SessionLogin:
    def __init__(self, session_id, username, password, session_id_version: str = "2.1"):
        self.session_id = session_id
        self.username = username
        self.password = password
        self.session_id_version = session_id_version



import pytest
from src.helpers import xmlBuilder
from src.models import SessionLogin


def test_serializeObject():
    sessionLogin = SessionLogin.SessionLogin("1", "blabla@bla.com", "blapass")

    serializedSession = xmlBuilder.serializeObject(sessionLogin)

    assert serializedSession == "<SessionLogin><sessionID>1</sessionID><password>blapass</password><userName>blabla@bla.com</userName><sessionIDVersion>2.1</sessionIDVersion></SessionLogin>"

def test_get_mac_address_of_interface():
    xmlData = """<NetworkInterfaceList version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                    <NetworkInterface version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                    <id>1</id>
                    <IPAddress version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                    <ipVersion>dual</ipVersion>
                    <addressingType>dynamic</addressingType>
                    <ipAddress>192.168.72.226</ipAddress>
                    <subnetMask>255.255.255.0</subnetMask>
                    <ipv6Address>fe80::2ea5:9cff:fecd:200a</ipv6Address>
                    <bitMask>ffff:ffff:ffff:ffff::</bitMask>
                    <ipV6AddressingType>dhcp</ipV6AddressingType>
                    <DefaultGateway>
                    <ipAddress>192.168.72.1</ipAddress>
                    </DefaultGateway>
                    <PrimaryDNS>
                    <ipAddress>192.168.72.1</ipAddress>
                    </PrimaryDNS>
                    <SecondaryDNS>
                    <ipAddress>0.0.0.0</ipAddress>
                    </SecondaryDNS>
                    </IPAddress>
                    <Discovery version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                    <UPnP>
                    <enabled>false</enabled>
                    </UPnP>
                    <Zeroconf>
                    <enabled>false</enabled>
                    </Zeroconf>
                    </Discovery>
                    <Link version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                    <MACAddress>2c:a5:9c:cd:20:0a</MACAddress>
                    <autoNegotiation>true</autoNegotiation>
                    <speed>0</speed>
                    <duplex>half</duplex>
                    <MTU min="500" max="9676">1500</MTU>
                    </Link>
                    </NetworkInterface>
                    <NetworkInterface version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                    <id>2</id>
                    <IPAddress version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                    <ipVersion>v4</ipVersion>
                    <addressingType>dynamic</addressingType>
                    <ipAddress>192.168.8.1</ipAddress>
                    <subnetMask>255.255.255.0</subnetMask>
                    <DefaultGateway>
                    <ipAddress>255.255.255.255</ipAddress>
                    </DefaultGateway>
                    <PrimaryDNS>
                    <ipAddress>0.0.0.0</ipAddress>
                    </PrimaryDNS>
                    <SecondaryDNS>
                    <ipAddress>0.0.0.0</ipAddress>
                    </SecondaryDNS>
                    </IPAddress>
                    <Link version="2.0" xmlns="http://www.hikvision.com/ver20/XMLSchema">
                    <MACAddress>60:1d:9d:b0:0f:fb</MACAddress>
                    <MTU min="500" max="9676">1500</MTU>
                    </Link>
                    </NetworkInterface>
                    </NetworkInterfaceList>"""

    mac_address = xmlBuilder.get_mac_address_of_interface(xmlData, 1)

    assert mac_address == '2c:a5:9c:cd:20:0a'


class IncorrectResponseContentError(Exception):
    def __init__(self):
        super().__init__("Response content is not in expected form.")    


class UnexpectedResponseCodeError(Exception):
    def __init__(self, response_code, response_text):
        super().__init__(f"Unexpected response status code {response_code} returned with message {response_text}")


import consts
from xml.etree import ElementTree


def serialize_object(obj):
    result = f"<{type(obj).__name__}>"
    
    for prop, value in vars(obj).items():
        if prop in consts.XML_SERIALIZABLE_NAMES:
            result += f"<{prop}>{value}</{prop}>"
    
    result += f"</{type(obj).__name__}>"

    return result


def get_mac_address_of_interface(xml_data, interface_id):
    try:
        root = ElementTree.fromstring(xml_data)
        namespaces = {'xmlns': consts.XML_SCHEMA}        
        for ni_element in root.findall('xmlns:NetworkInterface', namespaces):
            if ni_element.find('xmlns:id', namespaces).text == str(interface_id):
                link_elm = ni_element.find('xmlns:Link', namespaces)
                return link_elm.find('xmlns:MACAddress', namespaces).text
    except(Exception) as ex:
        return ''
    
    return ''
